using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;

namespace SPCAMLEditor
{
    public partial class SimpleXmlTextEditor : UserControl,IXmlTextEditor
    {
        public SimpleXmlTextEditor()
        {
            InitializeComponent();
            this.textBox1.MaxLength = 0;
        }

        private bool _NodeXmlLoaded = false ;

        private string _DescriptionAttributeName = "Name";
        public string DescriptionAttributeName
        {
            get { return _DescriptionAttributeName; }
            set { _DescriptionAttributeName = value; }
        }

        private XmlDocument _XmlDoc;
        private OpenFileDialog openFileDialog1;

        public XmlDocument XmlDoc
        {
            get {
                SyncXml();
                return _XmlDoc; 
            }
           // set { _XmlDoc = value; }
        }    

        private string _Xml;

        public string Xml
        {
            get 
            {
                SyncXml();
                return XmlDoc.DocumentElement.OuterXml;
                //return _Xml; 
            }
            set
            { 
                _Xml = value;

                if (this.DesignMode == true)
                    return;

                try
                {
                    if (!String.IsNullOrEmpty(_Xml))
                        ParserXml();
                }
                catch { }
            }
        }

        private string FormatXml(string source)
        {
            StringBuilder sb = new StringBuilder();
            XmlTextWriter writer = null;

            try
            {
                XmlDocument doc = new XmlDocument();
                doc.LoadXml(source);

                writer = new XmlTextWriter(new StringWriter(sb));
                writer.Formatting = Formatting.Indented;
                doc.WriteTo(writer);
            }
            finally
            {
                if (writer != null) writer.Close();
            }

            return sb.ToString();
        }

        void ParserXml()
        {           
            try
            {
                _XmlDoc = new XmlDocument();
                _XmlDoc.LoadXml(_Xml);

                this.treeView1.Nodes.Clear();

                BuildTree(_XmlDoc.DocumentElement, this.treeView1.Nodes);

                this._NodeXmlLoaded = false;
                this.textBox1.Text = this.FormatXml(_Xml);
                this._NodeXmlLoaded = true;

            }
            catch( Exception ex )
            {
                MessageBox.Show("Error" + ex.Message);
                this.textBox1.Text = _Xml;
                _XmlDoc = null;
            }
        }

        string GetNodeText(XmlNode xNode)
        {
            string text = xNode.Name;

             if (!String.IsNullOrEmpty(DescriptionAttributeName) &&
                xNode.Attributes[DescriptionAttributeName] != null)
            {
                text += "(" + xNode.Attributes[DescriptionAttributeName].Value + ")";
            }
            return text;
        }

        void BuildTree(XmlNode xNode , TreeNodeCollection nodes )
        {
            if (xNode.NodeType != XmlNodeType.Element) return;

            TreeNode rootNode = new TreeNode();
            rootNode.Tag = xNode;// path + "/" + xNode.Name;
            rootNode.Text = GetNodeText(xNode);            

            nodes.Add(rootNode);

            foreach (XmlNode cn in xNode.ChildNodes )
            {
                BuildTree(cn, rootNode.Nodes);
            }

            if (rootNode.Level == 0)
                rootNode.Expand();
        }

        private XmlNode _selectedXmlNode ;
        private TreeNode _selectedNode;
        private bool _innerXmlChanged = false;
        void SyncXml()
        {
            if (_selectedXmlNode == null || _innerXmlChanged == false ) return;

            string xml = this.textBox1.Text.Trim();

            if (xml == "")
            {
                _selectedNode.Remove();
                _selectedNode = null;
                _selectedXmlNode.ParentNode.RemoveChild(_selectedXmlNode);
            }
            else
            {
                _selectedNode.Collapse();
                _selectedNode.Nodes.Clear();

                XmlDocument doc = new XmlDocument();

                try
                {
                    doc.LoadXml(xml);
                }
                catch(Exception ex)
                {
                    MessageBox.Show( "Xml error" + ex.Message );
                    return;
                }
                _selectedXmlNode.InnerXml = doc.DocumentElement.InnerXml;
                _selectedXmlNode.Attributes.RemoveAll();

                foreach (XmlAttribute att in doc.DocumentElement.Attributes)
                {
                    XmlAttribute newAtt = _selectedXmlNode.OwnerDocument.CreateAttribute(att.Name);
                    newAtt.Value = att.Value;
                    _selectedXmlNode.Attributes.Append(newAtt);
                }

                _selectedNode.Text = this.GetNodeText(_selectedXmlNode);
                foreach (XmlNode cNode in _selectedXmlNode.ChildNodes)
                {
                    this.BuildTree(cNode, _selectedNode.Nodes);
                }
            }

            _innerXmlChanged = false;
        }

        private void treeView1_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            string nodePath = e.Node.FullPath.Replace("\\", "/");
            nodePath = ""+e.Node.Tag;

            SyncXml();

            if (_selectedNode != e.Node)
            {
               /// _selectedXmlNode = _XmlDoc.SelectSingleNode(nodePath);
                _selectedXmlNode = (XmlNode)e.Node.Tag;
                if (_selectedXmlNode != null)
                {
                    if (_selectedNode != null)
                        _selectedNode.BackColor = Color.Empty;
                    _selectedNode = e.Node;
                    _selectedNode.BackColor = Color.Yellow;

                    this.textBox1.TextChanged -= new EventHandler(textBox1_TextChanged);

                    this.textBox1.Text = this.FormatXml(_selectedXmlNode.OuterXml);

                    this.textBox1.TextChanged += new EventHandler(textBox1_TextChanged);
                }

            }
        }

        private void textBox1_TextChanged(object sender, EventArgs e)
        {
               _innerXmlChanged = true;
        }

        string GetInnerXml(string xml)
        {
            XmlDocument doc = new XmlDocument();
            doc.LoadXml(xml);
            return doc.DocumentElement.InnerXml;
        }

       
    }

   
     
     

}
